@@ -4,7 +4,7 @@
  * vim: tabstop=4 shiftwidth=4 noexpandtab :
  */
 
-#include "config.h"
+#include <config.h>
 
 #include <glib.h>
 
@@ -17,6 +17,10 @@
 #include <sys/wait.h>
 #endif
 
+#ifdef HAVE_LIBXML
+#include <libxml/tree.h>
+#endif
+
 #include <sys/types.h>
 
 /* FIXME: translate. */
@@ -203,6 +207,7 @@ lj_entry_load_metadata(LJEntry *entry,
 	return TRUE;
 }
 
+#ifdef HAVE_LIBXML
 static xmlNodePtr
 addtextchildenc(xmlDocPtr doc, xmlNodePtr parent, char *name, char *val) {
 	char *enc;
@@ -222,8 +227,8 @@ addtextchildenc(xmlDocPtr doc, xmlNodePtr parent, char *name, char *val) {
 	if (entry->A)                                 \
 		addtextchildenc(doc, root, #A, entry->A);
 
-xmlNodePtr
-lj_entry_to_xml_node(LJEntry *entry, xmlDocPtr doc) {
+void*
+lj_entry_to_xml_node(LJEntry *entry, void* doc) {
 	xmlNodePtr root, node;
 	root = xmlNewDocNode(doc, NULL, "entry", NULL);
 
@@ -287,7 +292,7 @@ lj_entry_to_xml_node(LJEntry *entry, xmlDocPtr doc) {
 	return root;
 }
 
-static xmlDocPtr
+static void*
 lj_entry_to_xml(LJEntry *entry) {
 	xmlDocPtr doc;
 	xmlNodePtr root;
@@ -298,6 +303,7 @@ lj_entry_to_xml(LJEntry *entry) {
 
 	return doc;
 }
+#endif /* HAVE_LIBXML */
 
 LJEntry*
 lj_entry_new_from_file(FILE *f, LJEntryFileType type,
@@ -435,6 +441,7 @@ lj_entries_new_from_result(LJResult *result, int count,
 	return entries;
 }
 
+#ifdef HAVE_LIBXML
 static void
 lj_entry_load_from_xml_node(LJEntry *entry, xmlDocPtr doc, xmlNodePtr node) {
 	char *itemid;
@@ -493,7 +500,7 @@ lj_entry_load_from_xml_node(LJEntry *entry, xmlDocPtr doc, xmlNodePtr node) {
 }
 
 LJEntry*
-lj_entry_new_from_xml_node(xmlDocPtr doc, xmlNodePtr node) {
+lj_entry_new_from_xml_node(void* doc, void* node) {
 	LJEntry *entry;
 	entry = lj_entry_new();
 	lj_entry_load_from_xml_node(entry, doc, node);
@@ -531,6 +538,7 @@ lj_entry_load_from_xml(LJEntry *entry, const char *data, int len, GError **err)
 	xmlFreeDoc(doc);
 	return TRUE;
 }
+#endif /* HAVE_LIBXML */
 
 static gboolean
 rfc822_get_keyval(const char **bufp, char *key, char *val) {
@@ -626,10 +634,12 @@ lj_entry_load_from_plain(LJEntry *entry, const char *data, int len) {
 
 static gboolean
 lj_entry_load_from_autodetect(LJEntry *entry, LJEntryFileType *typeret, const char *data, int len, GError **err) {
+#ifdef HAVE_LIBXML
 	if (lj_entry_load_from_xml(entry, data, len, err)) {
 		*typeret = LJ_ENTRY_FILE_XML;
 		return TRUE;
 	}
+#endif /* HAVE_LIBXML */
 	g_clear_error(err);
 	if (probe_rfc822(data)) {
 		lj_entry_load_from_rfc822(entry, data, len);
@@ -647,8 +657,10 @@ static gboolean
 lj_entry_load(LJEntry *entry, gchar *data, gsize len,
            LJEntryFileType type, LJEntryFileType *typeret, GError **err) {
 	switch (type) {
+#ifdef HAVE_LIBXML
 		case LJ_ENTRY_FILE_XML:
 			return lj_entry_load_from_xml(entry, data, len, err);
+#endif /* HAVE_LIBXML */
 		case LJ_ENTRY_FILE_RFC822:
 			lj_entry_load_from_rfc822(entry, data, len);
 			return TRUE;
@@ -661,6 +673,7 @@ lj_entry_load(LJEntry *entry, gchar *data, gsize len,
 	}
 }
 
+#ifdef HAVE_LIBXML
 gboolean
 lj_entry_to_xml_file(LJEntry *entry, const char *filename, GError **err) {
 	int ret;
@@ -678,6 +691,7 @@ lj_entry_to_xml_file(LJEntry *entry, const char *filename, GError **err) {
 	}
 	return TRUE;
 }
+#endif /* HAVE_LIBXML */
 
 /* the do/while kludge is to allow this macro to come in a blockless if */
 #define _FILE_ERROR_THROW(err, message)                          \
