@@ -13,6 +13,7 @@
 #endif
 
 #ifdef HAVE_REGEX_H
+#include <sys/types.h>
 #include <regex.h>
 #endif
 
@@ -52,12 +53,30 @@ typedef struct {
 } Cmdline;
 
 struct _Command {
-	char *cmdname; char *desc;
-	gboolean existinguser, needpw;
+	const char *cmdname; const char *desc;
+	gboolean requireuser, existinguser, needpw;
 	void (*action)(Cmdline *cmdline, JamAccount *acc, gint argc, gchar *argv[]);
 };
 
-static void command_dispatch(Cmdline *cmdline, Command *commands, gchar *help, int argc, gchar *argv[]);
+static void command_dispatch(Cmdline *cmdline, Command *commands, const char *help, int argc, gchar *argv[]);
+
+#ifndef HAVE_UNISTD_H
+/* this is a lame function. it isn't available on windows. */
+static gchar*
+getpass(const gchar* prompt) {
+	static gchar buf[64];
+	gint i;
+	g_print("%s", prompt);
+	for (i = 0; i < 64; i++) {
+		gint ch = getchar(); /* FIXME: this echoes */
+		if (ch == EOF || ch == (gint)'\n' || ch == (gint)'\r' /* win32 */)
+			break;
+		buf[i] = (gchar)ch;
+	}
+	buf[63] = '\0';
+	return buf;
+}
+#endif /* HAVE_UNISTD_H */
 
 static void
 print_header(void) {
@@ -121,8 +140,9 @@ print_command_help(Command *cmd, const char *helptext, Command *subcmds) {
 
 #define _GETOPT_LOOP(name)                                                  \
 {                                                                           \
-	char *short_options = name ## _SHORT_OPTIONS;                           \
+	const char *short_options = name ## _SHORT_OPTIONS;                     \
 	struct option long_options[] = name ## _LONG_OPTIONS;                   \
+	optind = 0;                                                             \
 	for (;;) {                                                              \
 		int c = getopt_long(argc, argv, short_options, long_options, NULL); \
 		if (c == -1) break;                                                 \
@@ -476,7 +496,7 @@ do_offline_summary(Cmdline *cmdline, JamAccount *acc, gint argc, gchar *argv[])
 
 	GETOPT_LOOP(OFFLINE_SUMMARY)
 	GETOPT_LOOP_END
-	command_dispatch(cmdline, NULL, OFFLINE_SUMMARY_HELP_TEXT, argc, argv);
+	command_dispatch(cmdline, NULL, _(OFFLINE_SUMMARY_HELP_TEXT), argc, argv);
 
 	if (!(js = journal_store_new(acc, &err))) {
 		fprintf(stderr,
@@ -504,7 +524,7 @@ do_offline_reindex(Cmdline *cmdline, JamAccount *acc, gint argc, gchar *argv[])
 
 	GETOPT_LOOP(OFFLINE_REINDEX)
 	GETOPT_LOOP_END
-	command_dispatch(cmdline, NULL, OFFLINE_REINDEX_HELP_TEXT, argc, argv);
+	command_dispatch(cmdline, NULL, _(OFFLINE_REINDEX_HELP_TEXT), argc, argv);
 
 	g_print(_("Rebuilding index..."));
 	fflush(stdout);
@@ -525,23 +545,59 @@ do_offline(Cmdline *cmdline, JamAccount *acc, gint argc, gchar *argv[]) {
 	print_command_help(cmdline->curcmd, _(OFFLINE_HELP_TEXT), commands);
 }
 
-#ifndef HAVE_UNISTD_H
-/* this is a lame function. it isn't available on windows. */
-static gchar*
-getpass(const gchar* prompt) {
-	static gchar buf[64];
-	gint i;
-	g_print("%s", prompt);
-	for (i = 0; i < 64; i++) {
-		gint ch = getchar(); /* FIXME: this echoes */
-		if (ch == EOF || ch == (gint)'\n' || ch == (gint)'\r' /* win32 */)
+
+static void
+do_user_add(Cmdline *cmdline, JamAccount *acc, gint argc, gchar *argv[]) {
+	const char *username, *password = NULL;
+	gboolean remember_password = TRUE;
+	JamHost *host;
+
+	GETOPT_LOOP(USER_ADD)
+		case 'p':
+			remember_password = FALSE;
 			break;
-		buf[i] = (gchar)ch;
+	GETOPT_LOOP_END
+
+	argc -= optind;
+	argv += optind;
+
+	if (argc < 1) {
+		g_print(_("Error: specify a user.\n"));
+		g_print(_(USER_ADD_HELP_TEXT));
+		exit(EXIT_FAILURE);
 	}
-	buf[63] = '\0';
-	return buf;
+	username = argv[0];
+
+	if (remember_password) {
+		if (argc < 2) {
+			password = getpass("Password: ");
+		} else {
+			password = argv[1];
+		}
+	}
+
+	conf_verify_a_host_exists();
+	if (conf.lasthost) {
+		host = conf.lasthost;
+	} else {
+		host = conf.hosts->data;
+		conf.lasthost = host;
+	}
+
+	acc = jam_host_get_account_by_username(host, username, /*create = */TRUE);
+	if (password)
+		jam_account_set_password(acc, password);
+	jam_account_set_remember(acc, TRUE, remember_password);
+	conf_write(&conf, app.conf_dir);
+	exit(EXIT_SUCCESS);
+}
+
+static void
+do_user(Cmdline *cmdline, JamAccount *acc, gint argc, gchar *argv[]) {
+	GETOPT_LOOP_SUBCOMMANDS(USER)
+	GETOPT_LOOP_SUBCOMMANDS_END(USER)
+	print_command_help(cmdline->curcmd, _(USER_HELP_TEXT), commands);
 }
-#endif /* HAVE_UNISTD_H */
 
 JamAccount*
 cmdline_load_account(Cmdline *cmdline, gboolean existinguser, gboolean needpw) {
@@ -607,7 +663,7 @@ cmdline_load_file(JamDoc *doc, char *filename, GError **err) {
 }
 
 static void
-command_dispatch(Cmdline *cmdline, Command *commands, gchar *help, int argc, gchar *argv[]) {
+command_dispatch(Cmdline *cmdline, Command *commands, const char *help, int argc, gchar *argv[]) {
 	JamAccount *acc;
 	Command *command = NULL;
 	char *cmdname;
@@ -617,27 +673,32 @@ command_dispatch(Cmdline *cmdline, Command *commands, gchar *help, int argc, gch
 	argv += optind;
 	cmdname = argv[0];
 
-	if (argc == 0)
+	if (argc <= 0)
 		return;
 
 	if (g_ascii_strcasecmp(cmdname, "help") == 0) {
-		g_print(_(help));
+		g_print(help);
 		exit(EXIT_SUCCESS);
 	}
 	for (i = 0; commands && commands[i].cmdname; i++) {
-		if (g_ascii_strcasecmp(cmdname, commands[i].cmdname) == 0)
+		if (g_ascii_strcasecmp(cmdname, commands[i].cmdname) == 0) {
 			command = &commands[i];
+			break;
+		}
 	}
 	if (!command) {
 		g_printerr(_("Error: Unknown action '%s'.\n"), cmdname);
 		exit(EXIT_FAILURE);
 	}
 	cmdline->curcmd = command;
-	acc = cmdline_load_account(cmdline, command->existinguser, command->needpw);
-	jam_doc_set_account(cmdline->doc, acc);
-	if (!acc) {
-		g_printerr(_("Error:  Must specify account.\n"));
-		exit(EXIT_FAILURE);
+	if (command->requireuser) {
+		acc = cmdline_load_account(cmdline,
+		                           command->existinguser, command->needpw);
+		if (!acc) {
+			g_printerr(_("Error:  Must specify account.\n"));
+			exit(EXIT_FAILURE);
+		}
+		jam_doc_set_account(cmdline->doc, acc);
 	}
 	command->action(cmdline, acc, argc, argv);
 	/* should terminate. */
