@@ -1,26 +1,44 @@
 /* logjam - a GTK client for LiveJournal.
- * Copyright (C) 2000-2002 Evan Martin <evan@livejournal.com>
+ * Copyright (C) 2000-2003 Evan Martin <evan@livejournal.com>
  *
  * vim: tabstop=4 shiftwidth=4 noexpandtab :
- * $Id: console.c,v 1.9 2002/10/02 02:04:27 martine Exp $
  */
+
 #include <config.h>
 
-#include <gtk/gtk.h>
+#include "gtk-all.h"
+
+#include <livejournal/consolecommand.h>
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdarg.h>
 #include "util.h"
 #include "network.h"
+#include "console.h"
 
 typedef struct {
 	GtkWidget *win;
 	GtkWidget *entry, *display;
 	GtkTextMark *mark_end;
+	JamAccountLJ *account;
 } ConsoleUI;
 
-static void
-console_print(ConsoleUI *cui, const char *type, const char *text, ...) {
+static char *
+name_from_type(LJConsoleLineType type) {
+	switch (type) {
+	case LJ_CONSOLE_LINE_TYPE_INFO:
+		return "info";
+	case LJ_CONSOLE_LINE_TYPE_ERROR:
+		return "error";
+	case LJ_CONSOLE_LINE_TYPE_UNKNOWN:
+	default:
+		return NULL;
+	}
+}
+
+void
+console_print(ConsoleUI *cui, char *type, const char *text, ...) {
 	char buf[4096];
 	va_list ap;
 	GtkTextBuffer *buffer;
@@ -42,43 +60,35 @@ console_print(ConsoleUI *cui, const char *type, const char *text, ...) {
 			cui->mark_end, 0.0, FALSE, 0, 0);
 }
 
-static gboolean
-run_command(ConsoleUI *cui, const char *command) {
-	NetRequest *request;
-	NetResult *result;
-	char *linetext, *linetype;
-	int line, linecount;
+gboolean
+run_console_command(ConsoleUI *cui, const char *command) {
+	NetContext *ctx;
+	LJConsoleCommand *cc;
+	int i;
 
 	console_print(cui, NULL, _("Running command: '"));
 	console_print(cui, "command", command);
 	console_print(cui, NULL, "'.\n");
 
-	request = net_request_new("consolecommand");
-	net_request_copys(request, "command", command);
-	result = net_request_run(cui->win, _("Running Command..."), request);
-	net_request_free(request);
+	cc = lj_consolecommand_new(jam_account_lj_get_user(cui->account), command);
 
-	if (!net_result_succeeded(result)) {
-		console_print(cui, _("error"), _("Error running command.\n"));
-		net_result_free(result);
+	ctx = net_ctx_gtk_new(GTK_WINDOW(cui->win), _("Running Command"));
+	if (!net_run_verb_ctx((LJVerb*)cc, ctx, NULL)) {
+		console_print(cui, "error", _("Error running command.\n"));
+		lj_consolecommand_free(cc);
+		net_ctx_gtk_free(ctx);
 		return FALSE;
 	}
 
-	linecount = net_result_geti(result, "cmd_line_count");
-
-	for (line = 1; line <= linecount; line++) {
-		char buf[30];
-		g_snprintf(buf, 30, "cmd_line_%d", line);
-		linetext = net_result_get(result, buf);
-		linetype = net_result_get_prefix(result, buf, "_type");
-		if (linetype == NULL)
-			linetype = "info";
-
-		console_print(cui, linetype, linetext);
-		console_print(cui, linetype, "\n");
+	for (i = 0; i < cc->linecount; i++) {
+		char *typename = name_from_type(cc->lines[i].type);
+		console_print(cui, typename, cc->lines[i].text);
+		console_print(cui, typename, "\n");
 	}
+
+	lj_consolecommand_free(cc);
+	net_ctx_gtk_free(ctx);
 	
-	net_result_free(result);
 	return TRUE;
 }
 
@@ -87,25 +97,32 @@ submit_cb(GtkWidget *w, ConsoleUI *cui) {
 	char *command;
 
 	command = gtk_editable_get_chars(GTK_EDITABLE(cui->entry), 0, -1);
-	if (run_command(cui, command))
+	if (run_console_command(cui, command))
 		gtk_editable_delete_text(GTK_EDITABLE(cui->entry), 0, -1);
 	g_free(command);
 }
 
 static void
 console_help(GtkWidget *w, ConsoleUI *cui) {
-	run_command(cui, "help");
+	run_console_command(cui, "help");
 }
 
 void
-console_dialog_run(GtkWidget *parent) {
+console_dialog_run(GtkWindow *parent, JamAccountLJ *acc) {
 	ConsoleUI *cui;
 	GtkWidget *vbox, *hbox, *button;
 	GtkTextBuffer *buffer;
 	GtkTextIter end;
 
 	cui = g_new0(ConsoleUI, 1);
-	cui->win = lj_dialog_new(parent, _("LiveJournal Console"), 300, 300);
+	cui->account = acc;
+	cui->win = gtk_dialog_new_with_buttons(_("LiveJournal Console"),
+			parent, GTK_DIALOG_DESTROY_WITH_PARENT,
+			GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
+			NULL);
+	jam_win_set_size(GTK_WINDOW(cui->win), 500, 400);
+	g_signal_connect_swapped(G_OBJECT(cui->win), "response",
+			G_CALLBACK(gtk_widget_destroy), cui->win);
 	g_signal_connect_swapped(G_OBJECT(cui->win), "destroy",
 			G_CALLBACK(g_free), cui);
 	geometry_tie(cui->win, GEOM_CONSOLE);
@@ -152,11 +169,9 @@ console_dialog_run(GtkWidget *parent) {
 			"foreground", "red",
 			NULL);
 
-	lj_dialog_add_close(cui->win);
-
-	lj_dialog_set_contents(cui->win, vbox);
+	jam_dialog_set_contents(GTK_DIALOG(cui->win), vbox);
 
 	console_print(cui, NULL, _("LiveJournal console ready.\n"));
 
-	lj_dialog_show(cui->win);
+	gtk_widget_show(cui->win);
 }
