@@ -1,26 +1,31 @@
 /* logjam - a GTK client for LiveJournal.
- * Copyright (C) 2000-2002 Evan Martin <evan@livejournal.com>
+ * Copyright (C) 2000-2003 Evan Martin <evan@livejournal.com>
  *
  * vim: tabstop=4 shiftwidth=4 noexpandtab :
- * $Id: friendedit.c,v 1.6 2002/09/24 07:57:50 martine Exp $
  */
 
-#include <gtk/gtk.h>
+#include "config.h"
+#include "gtk-all.h"
 
 #include <stdlib.h>
 #include <string.h>
 
+#include <livejournal/editfriends.h>
+
 #include "friends.h"
-#include "friendedit.h"
 #include "network.h"
 #include "conf.h"
 #include "util.h"
+#include "account.h"
+
+#include "friendedit.h"
 
 typedef struct {
 	GtkWidget *win;
 	GtkWidget *eusername, *ebgcolor, *efgcolor;
 
-	Friend *editfriend;
+	LJFriend *editfriend;
+	JamAccountLJ *account;
 } FriendEditUI;
 
 static void
@@ -50,7 +55,6 @@ change_entry_color_dlg(FriendEditUI *feui, GtkWidget *toedit, const char *title)
 
 	dlg = gtk_color_selection_dialog_new(title);
 	gtk_window_set_transient_for(GTK_WINDOW(dlg), GTK_WINDOW(feui->win));
-	lj_win_set_icon(dlg);
 	gtk_widget_hide(GTK_COLOR_SELECTION_DIALOG(dlg)->help_button);
 
 	csel = GTK_COLOR_SELECTION(GTK_COLOR_SELECTION_DIALOG(dlg)->colorsel);
@@ -63,14 +67,14 @@ change_entry_color_dlg(FriendEditUI *feui, GtkWidget *toedit, const char *title)
 		gtk_color_selection_set_current_color(csel, &color); 
 	}
 
-	if (lj_dialog_run(dlg)) {
+	if (gtk_dialog_run(GTK_DIALOG(dlg)) == GTK_RESPONSE_OK) {
 		gtk_color_selection_get_current_color(csel, &color);
 
 		gdkcolor_to_hex(&color, new_hex);
 
 		gtk_entry_set_text(GTK_ENTRY(toedit), new_hex);
-		gtk_widget_destroy(dlg);
 	}
+	gtk_widget_destroy(dlg);
 
 	return 0;
 }
@@ -87,109 +91,105 @@ change_col_fg(GtkWidget *w, FriendEditUI *feui) {
 
 static gboolean 
 add_the_friend(FriendEditUI *feui) {
-	NetRequest *request;
-	NetResult *result;
-	int friends_added = 0;
+	NetContext *ctx;
+	LJEditFriends *ef;
 	gchar *username, *name;
 
-	request = net_request_new("editfriends");
+	ctx = net_ctx_gtk_new(GTK_WINDOW(feui->win), _("Adding Friend"));
 
-	net_request_sets(request, "editfriend_add_1_user",
-			g_strdup(gtk_entry_get_text(GTK_ENTRY(feui->eusername))));
-	net_request_sets(request, "editfriend_add_1_fg",
-			g_strdup(gtk_entry_get_text(GTK_ENTRY(feui->efgcolor))));
-	net_request_sets(request, "editfriend_add_1_bg",
-			g_strdup(gtk_entry_get_text(GTK_ENTRY(feui->ebgcolor))));
-
-	result = net_request_run(feui->win, _("Adding Friend..."), request);
-	hash_destroy(request);
-
-	if (!net_result_succeeded(result)) {
-		hash_destroy(result);
+	ef = lj_editfriends_new(jam_account_lj_get_user(feui->account));
+	lj_editfriends_add_friend(ef,
+			gtk_entry_get_text(GTK_ENTRY(feui->eusername)),
+			gtk_entry_get_text(GTK_ENTRY(feui->efgcolor)),
+			gtk_entry_get_text(GTK_ENTRY(feui->ebgcolor)));
+			
+	if (!net_run_verb_ctx((LJVerb*)ef, ctx, NULL) || ef->addcount != 1) {
+		lj_editfriends_free(ef);
+		net_ctx_gtk_free(ctx);
 		return FALSE;
 	}
 
-	friends_added = net_result_geti(result, "friends_added");
-	if (friends_added != 1) {
-		hash_destroy(result);
-		return FALSE;
-	}
-	
-	name = net_result_get(result, "friend_1_name");
-	username = net_result_get(result, "friend_1_user");
+	name = ef->added[0].fullname;
+	username = ef->added[0].username;
 
 	if (feui->editfriend == NULL || 
 			strcmp(feui->editfriend->username, username) != 0) {
 		/* we must create a new friend */
-		feui->editfriend = friend_new();
-		feui->editfriend->conn = FRIEND_CONN_MY;
+		feui->editfriend = lj_friend_new();
+		feui->editfriend->conn = LJ_FRIEND_CONN_MY;
 	}
 
 	string_replace(&feui->editfriend->username, g_strdup(username));
 	string_replace(&feui->editfriend->fullname, g_strdup(name));
-	strcpy(feui->editfriend->foreground,
+	feui->editfriend->foreground = lj_color_to_int(
 			gtk_entry_get_text(GTK_ENTRY(feui->efgcolor)));
-	strtoupper(feui->editfriend->foreground);
-	strcpy(feui->editfriend->background,
+	feui->editfriend->background = lj_color_to_int(
 			gtk_entry_get_text(GTK_ENTRY(feui->ebgcolor)));
-	strtoupper(feui->editfriend->background);
 
-	gtk_widget_destroy(feui->win);
-
-	hash_destroy(result);
+	lj_editfriends_free(ef);
+	net_ctx_gtk_free(ctx);
 	return TRUE;
 }
 
 static void
-entry_changed(GtkEntry *entry, GtkWidget* button) {
-	gtk_widget_set_sensitive(button, 
+entry_changed(GtkEntry *entry, FriendEditUI* feui) {
+	gtk_dialog_set_response_sensitive(GTK_DIALOG(feui->win),
+			GTK_RESPONSE_OK, 
 			(strlen(gtk_entry_get_text(entry)) > 0));
 }
 
-static void
-dialog_ok_cb(GtkEntry *entry, FriendEditUI* feui) {
-	gtk_dialog_response(GTK_DIALOG(feui->win), GTK_RESPONSE_OK);
-}
-
-Friend*
-friend_edit_dlg_run(GtkWidget *parent, gboolean edit, Friend *f) {
+LJFriend*
+friend_edit_dlg_run(GtkWindow *parent, JamAccountLJ *acc, gboolean edit, LJFriend *f) {
 	FriendEditUI feui_actual = { 0 };
 	FriendEditUI *feui = &feui_actual;
 	GtkWidget *button;
 	GtkWidget *table;
 
-	memset(feui, 0, sizeof(FriendEditUI));
+	feui->account = acc;
 	feui->editfriend = f;
 
-	feui->win = lj_dialog_new(parent,
+	feui->win = gtk_dialog_new_with_buttons(
 			edit ?  _("Edit Friend") :
 			(f ? _("Add This Friend") : _("Add a Friend")),
-			-1, -1);
+			parent, GTK_DIALOG_MODAL,
+			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+			edit ? _("Change") : _("Add"), GTK_RESPONSE_OK,
+			NULL);
+	gtk_dialog_set_default_response(GTK_DIALOG(feui->win), GTK_RESPONSE_OK);
 
-	table = lj_table_new(3, 3);
+	table = jam_table_new(3, 3);
 	   
 	/* make the labels/entries */
 	feui->eusername = gtk_entry_new();
+	gtk_entry_set_activates_default(GTK_ENTRY(feui->eusername), TRUE);
 	gtk_entry_set_max_length(GTK_ENTRY(feui->eusername), 18);
-	if (edit) gtk_editable_set_editable(GTK_EDITABLE(feui->eusername), FALSE);
+	if (edit)
+		gtk_editable_set_editable(GTK_EDITABLE(feui->eusername), FALSE);
+	else
+		/* enable/disable the button based on name text */
+		g_signal_connect(G_OBJECT(feui->eusername), "changed",
+			G_CALLBACK(entry_changed), feui);
+
 	gtk_widget_set_size_request(feui->eusername, 100, -1);
-	lj_table_label_content(GTK_TABLE(table), 0, 
+	jam_table_label_content(GTK_TABLE(table), 0, 
 			_("Friend's _username:"), feui->eusername);
 
 	feui->efgcolor = gtk_entry_new();
+	gtk_entry_set_activates_default(GTK_ENTRY(feui->efgcolor), TRUE);
 	gtk_entry_set_max_length(GTK_ENTRY(feui->efgcolor), 7);
 	g_signal_connect(G_OBJECT(feui->efgcolor), "changed",
 			G_CALLBACK(color_entry_changed), feui);
 	gtk_widget_set_size_request(feui->efgcolor, 100, -1);
-	lj_table_label_content(GTK_TABLE(table), 1, 
+	jam_table_label_content(GTK_TABLE(table), 1, 
 			_("_Text color:"), feui->efgcolor);
 
 	feui->ebgcolor = gtk_entry_new();
+	gtk_entry_set_activates_default(GTK_ENTRY(feui->ebgcolor), TRUE);
 	gtk_entry_set_max_length(GTK_ENTRY(feui->ebgcolor), 7);
 	g_signal_connect(G_OBJECT(feui->ebgcolor), "changed",
 			G_CALLBACK(color_entry_changed), feui);
 	gtk_widget_set_size_request(feui->ebgcolor, 100, -1);
-	lj_table_label_content(GTK_TABLE(table), 2,
+	jam_table_label_content(GTK_TABLE(table), 2,
 			_("_Background color:"), feui->ebgcolor);
 
 	/* make the color selector buttons */
@@ -202,37 +202,34 @@ friend_edit_dlg_run(GtkWidget *parent, gboolean edit, Friend *f) {
 	g_signal_connect(G_OBJECT(button), "clicked",
 			G_CALLBACK(change_col_bg), feui);
 		
-	lj_dialog_set_contents(feui->win, table);
-
-	button = lj_dialog_add_okcancel(feui->win, edit ? _("  Change  ") : _("  Add  ")),
-
-	/* enable/disable the button based on name text */
-	g_signal_connect(G_OBJECT(feui->eusername), "changed",
-		G_CALLBACK(entry_changed), button);
-
-	g_signal_connect(G_OBJECT(feui->eusername), "activate",
-		G_CALLBACK(dialog_ok_cb), feui);
+	jam_dialog_set_contents(GTK_DIALOG(feui->win), table);
 
 	/* fill in default values. */
 	if (f) {
 		gtk_entry_set_text(GTK_ENTRY(feui->eusername), f->username);
 	} else {
-		gtk_entry_set_text(GTK_ENTRY(feui->eusername), "");
-		/* emit the "changed" signal, anyway. */
+		/* emit the "changed" signal, in any case. */
+		g_signal_emit_by_name(G_OBJECT(feui->eusername), "changed");
 	}
 
 	if (!edit) {
 		gtk_entry_set_text(GTK_ENTRY(feui->efgcolor), "#000000");
 		gtk_entry_set_text(GTK_ENTRY(feui->ebgcolor), "#FFFFFF");
 	} else {
-		gtk_entry_set_text(GTK_ENTRY(feui->efgcolor), f->foreground);
-		gtk_entry_set_text(GTK_ENTRY(feui->ebgcolor), f->background);
+		char color[10];
+		lj_int_to_color(f->foreground, color);
+		gtk_entry_set_text(GTK_ENTRY(feui->efgcolor), color);
+		lj_int_to_color(f->background, color);
+		gtk_entry_set_text(GTK_ENTRY(feui->ebgcolor), color);
 	}
 
-	while (lj_dialog_run(feui->win)) {
-		if (add_the_friend(feui))
+	while (gtk_dialog_run(GTK_DIALOG(feui->win)) == GTK_RESPONSE_OK) {
+		if (add_the_friend(feui)) {
+			gtk_widget_destroy(feui->win);
 			return feui->editfriend;
+		}
 	}
+	gtk_widget_destroy(feui->win);
 	return NULL;
 }
 
