@@ -1,13 +1,12 @@
 /* logjam - a GTK client for LiveJournal.
- * Copyright (C) 2000-2002 Evan Martin <evan@livejournal.com>
+ * Copyright (C) 2000-2003 Evan Martin <evan@livejournal.com>
  *
  * vim: tabstop=4 shiftwidth=4 noexpandtab :
- * $Id: friends.c,v 1.29 2002/11/10 08:30:12 martine Exp $
  */
 
 #include "config.h"
 
-#include <gtk/gtk.h>
+#include "gtk-all.h"
 #include <time.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -15,29 +14,36 @@
 #include <ctype.h>
 #include <errno.h>
 
-#include "friends.h"
+#include <livejournal/livejournal.h>
+#include <livejournal/getfriends.h>
+#include <livejournal/editfriends.h>
+
 #include "network.h"
 #include "conf.h"
 #include "util.h"
 #include "spawn.h"
+#include "account.h"
 #include "friends.h"
 #include "friendedit.h"
 #include "friendgroups.h"
 
 #include "icons.h"
 
+#include "friends.h"
+
 typedef struct {
 	GtkWindow  win; /* parent */
-	GtkWidget *friendview, *statspanel, *filterpanel, *statslabel, *statscomm;
+	GtkWidget *friendview, *statspanel, *statslabel, *statscomm;
 	GtkItemFactory *itemfactory;
 	GtkWidget *omconn, *omtype;
 
 	GdkPixbuf *pb_user, *pb_comm, *pb_larrow, *pb_rarrow, *pb_lrarrow;
 
-	GList *friends;
-	FriendType filter_type;
+	JamAccountLJ *account;
+	GSList *friends;
+	LJFriendType filter_type;
 	int filter_conn;
-} FriendsUI;
+} JamFriendsUI;
 
 enum {
 	FRIEND_COL_LINK,
@@ -53,11 +59,10 @@ enum {
 	ACTION_REMOVE,
 	ACTION_JOURNAL_VIEW,
 	ACTION_USERINFO_VIEW,
-	ACTION_STATISTICS,
-	ACTION_FILTER
+	ACTION_HIDE_STATISTICS
 };
 
-#define FRIENDSUI(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj), friendsui_get_type(), FriendsUI))
+#define FRIENDSUI(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj), friendsui_get_type(), JamFriendsUI))
 
 static GType
 friendsui_get_type(void) {
@@ -70,28 +75,28 @@ friendsui_get_type(void) {
 			NULL,
 			NULL,
 			NULL,
-			sizeof (FriendsUI),
+			sizeof (JamFriendsUI),
 			0,
 			NULL
 		};
 		new_type = g_type_register_static(GTK_TYPE_WINDOW,
-				"FriendsUI", &new_info, 0);
+				"JamFriendsUI", &new_info, 0);
 	}
 	return new_type;
 }
 
 static GtkWidget*
 friendsui_new(void) {
-	FriendsUI *fui = FRIENDSUI(g_object_new(friendsui_get_type(), NULL));
+	JamFriendsUI *fui = FRIENDSUI(g_object_new(friendsui_get_type(), NULL));
 	return GTK_WIDGET(fui);
 }
 
-static gboolean request_remove_friend(FriendsUI *fui, const char *username);
+static gboolean request_remove_friend(JamFriendsUI *fui, const char *username);
 
 static void
-recalculate_stats(FriendsUI *fui) {
-	GList *l;
-	Friend *f;
+recalculate_stats(JamFriendsUI *fui) {
+	GSList *l;
+	LJFriend *f;
 	char buf[1024];
 	gboolean includecomm;
 
@@ -103,13 +108,13 @@ recalculate_stats(FriendsUI *fui) {
 	for (l = fui->friends; l; l = l->next) {
 		f = l->data;
 
-		if (!includecomm && f->type == FRIEND_TYPE_COMMUNITY)
+		if (!includecomm && f->type == LJ_FRIEND_TYPE_COMMUNITY)
 			continue;
 		
 		switch (f->conn) {
-			case FRIEND_CONN_MY:
+			case LJ_FRIEND_CONN_MY:
 				fmcount++; break;
-			case FRIEND_CONN_OF:
+			case LJ_FRIEND_CONN_OF:
 				focount++; break;
 			default:
 				break;
@@ -122,6 +127,7 @@ recalculate_stats(FriendsUI *fui) {
 			"Inclusive:\n"
 			"Friends: %d\n"
 			"Friend Of: %d\n"
+			"Ratio: %d%%\n"
 			"\n"
 			"Exclusive:\n"
 			"Friends: %d\n"
@@ -129,14 +135,15 @@ recalculate_stats(FriendsUI *fui) {
 			"Friend Both: %d\n"),
 			count,
 			count-focount, count-fmcount,
+			((count-fmcount) != 0) ? ((count-focount)*100)/(count-fmcount) : 0,
 			fmcount, focount, count-fmcount-focount);
 	gtk_label_set_markup(GTK_LABEL(fui->statslabel), buf);
 }
 
-static Friend*
-friend_exists(FriendsUI *fui, char *username) {
-	GList *l;
-	Friend *f;
+static LJFriend*
+friend_exists(JamFriendsUI *fui, char *username) {
+	GSList *l;
+	LJFriend *f;
 
 	for (l = fui->friends; l; l = l->next) {
 		f = l->data;
@@ -149,87 +156,40 @@ friend_exists(FriendsUI *fui, char *username) {
 
 static gint
 friend_compare(gconstpointer a, gconstpointer b) {
-	return g_ascii_strcasecmp(((Friend*)a)->username, ((Friend*)b)->username);
-}
-
-static int 
-load_friends(FriendsUI *fui, GtkWidget *mainwin) {
-	NetRequest *request;
-	NetResult *result;
-	int i, friend_count, friendof_count;
-	char *username, *value;
-	guint32 mask;
-	Friend *f;
-	char buf[30];
-	
-	request = net_request_new("getfriends");
-	net_request_seti(request, "includefriendof", 1);
-	result = net_request_run(mainwin, _("Loading Friends..."), request);
-	net_request_free(request);
-
-	if (!net_result_succeeded(result)) {
-		net_result_free(result);
-		return -1;
-	}
-
-	friend_count = net_result_geti(result, "friend_count");
-	for (i = 1; i <= friend_count; i++) {
-		sprintf(buf, "friend_%d_", i);
-		f = friend_new();
-		f->username = g_strdup(net_result_get_prefix(result, buf, "user"));
-		f->fullname = g_strdup(net_result_get_prefix(result, buf, "name"));
-
-		value = net_result_get_prefix(result, buf, "fg");
-		if (value) strcpy(f->foreground, value);
-		strtoupper(f->foreground);
-		value = net_result_get_prefix(result, buf, "bg");
-		if (value) strcpy(f->background, value);
-		strtoupper(f->background);
-
-		value = net_result_get_prefix(result, buf, "groupmask");
-		if (value) {
-			mask = atoi(value);
-		} else {
-			mask = FRIEND_GROUP_ALLFRIENDS;
-		}
-		f->groupmask = mask;
-
-		f->conn = FRIEND_CONN_MY;
-		f->type = friend_type_from_str(
-				net_result_get_prefix(result, buf, "type"));
-
-		fui->friends = g_list_insert_sorted(fui->friends, f, friend_compare);
-	}	   
-
-	friendof_count = net_result_geti(result, "friendof_count");
-	for (i = 1; i <= friendof_count; i++) {
-		sprintf(buf, "friendof_%d_", i);
-		username = net_result_get_prefix(result, buf, "user");
-
-		f = friend_exists(fui, username);
-		if (f) {
-			f->conn |= FRIEND_CONN_OF;
-		} else {
-			f = friend_new();
-			f->username = g_strdup(username);
-			f->fullname = g_strdup(net_result_get_prefix(result, buf, "name"));
-			f->type = friend_type_from_str(net_result_get_prefix(result, buf,
-						"type"));
-			f->conn = FRIEND_CONN_OF;
-
-			fui->friends = g_list_insert_sorted(fui->friends, f, friend_compare);
-		}
-	}	   
-
-	net_result_free(result);
-	return 0;
+	return g_ascii_strcasecmp(((LJFriend*)a)->username, ((LJFriend*)b)->username);
 }
 
 static void
-populate_model(FriendsUI *fui) {
+friends_hash_list_cb(gpointer key, LJFriend *f, GSList **list) {
+	*list = g_slist_insert_sorted(*list, f, friend_compare);
+}
+
+static gboolean
+load_friends(GtkWindow *parent, JamAccountLJ *acc, GSList **l) {
+	LJGetFriends *getfriends;
+	NetContext *ctx;
+	
+	ctx = net_ctx_gtk_new(parent, _("Loading Friends"));
+	getfriends = lj_getfriends_new(jam_account_lj_get_user(acc));
+	if (!net_run_verb_ctx((LJVerb*)getfriends, ctx, NULL)) { 
+		lj_getfriends_free(getfriends, TRUE);
+		net_ctx_gtk_free(ctx);
+		return FALSE;
+	}
+
+	g_hash_table_foreach(getfriends->friends, (GHFunc)friends_hash_list_cb, l);
+
+	lj_getfriends_free(getfriends, FALSE);
+	net_ctx_gtk_free(ctx);
+
+	return TRUE;
+}
+
+static void
+populate_model(JamFriendsUI *fui) {
 	GtkListStore *liststore;
-	GList *l;
-	Friend *f;
+	GSList *l;
+	LJFriend *f;
 	GtkTreeIter iter;
 	
 	liststore = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(fui->friendview)));
@@ -251,29 +211,28 @@ populate_model(FriendsUI *fui) {
 	}
 }
 
-static void close_cb         (FriendsUI *fui);
-static void friend_add_cb    (FriendsUI *fui);
-static void friend_edit_cb   (FriendsUI *fui);
-static void friend_remove_cb (FriendsUI *fui);
-static void friend_journal_view_cb(FriendsUI *fui);
-static void friend_userinfo_view_cb(FriendsUI *fui);
-static void friendgroups_cb  (FriendsUI *fui);
-static void stats_show_cb    (FriendsUI *fui,
+static void close_cb         (JamFriendsUI *fui);
+static void friend_add_cb    (JamFriendsUI *fui);
+static void friend_edit_cb   (JamFriendsUI *fui);
+static void friend_remove_cb (JamFriendsUI *fui);
+static void friend_journal_view_cb(JamFriendsUI *fui);
+static void friend_userinfo_view_cb(JamFriendsUI *fui);
+static void friendgroups_cb  (JamFriendsUI *fui);
+static void stats_cb         (JamFriendsUI *fui,
                               gint action, GtkCheckMenuItem *item);
-static void filter_show_cb   (FriendsUI *fui,
-                              gint action, GtkCheckMenuItem *item);
-static void export_cb        (FriendsUI *fui);
+static void export_cb        (JamFriendsUI *fui);
 
 static GtkWidget*
-make_menu(FriendsUI *fui) {
+make_menu(JamFriendsUI *fui) {
 	GtkWidget *bar;
 	GtkAccelGroup *accelgroup = NULL;
 	GtkItemFactory *item_factory = NULL;
 
 static GtkItemFactoryEntry menu_items[] = {
-	{ N_("/_Friends/Friend Groups..."), NULL, friendgroups_cb, 0, NULL },
-	{ N_("/_Friends/sep"),              NULL, NULL, 0, "<Separator>" },
-	{ N_("/_Friends/_Close"),     NULL, close_cb, 0,
+	{ N_("/_Friends"),               NULL, NULL, 0, "<Branch>" },
+	{ N_("/Friends/_Friend Groups..."), NULL, friendgroups_cb, 0, NULL },
+	{ N_("/Friends/sep"),              NULL, NULL, 0, "<Separator>" },
+	{ N_("/Friends/_Close"),     NULL, close_cb, 0,
 	                           "<StockItem>", GTK_STOCK_CLOSE },
 
 	{ N_("/_Edit"),                 NULL, NULL, ACTION_EDIT_MENU, "<Branch>" },
@@ -290,14 +249,14 @@ static GtkItemFactoryEntry menu_items[] = {
 	                             "<StockItem>", GTK_STOCK_JUMP_TO },
 
 	{ N_("/_Tools"),                 NULL, NULL, 0, "<Branch>" },
-	{ N_("/Tools/Show _Statistics"), NULL, stats_show_cb, ACTION_STATISTICS, "<CheckItem>" },
-	{ N_("/Tools/Show _Filter"),     NULL, filter_show_cb, ACTION_FILTER, "<CheckItem>" },
+	{ N_("/Tools/Hide _Statistics"), NULL, stats_cb, ACTION_HIDE_STATISTICS, "<CheckItem>" },
 	{ N_("/Tools/Export..."),        NULL, export_cb, 0, NULL },
 };
 	int itemcount = sizeof(menu_items) / sizeof(menu_items[0]);
 
 	accelgroup = gtk_accel_group_new();
 	item_factory = gtk_item_factory_new(GTK_TYPE_MENU_BAR, "<main>", accelgroup);
+	gtk_item_factory_set_translate_func(item_factory, gettext_translate_func, NULL, NULL);
 	gtk_item_factory_create_items(item_factory, itemcount, menu_items, fui);
 	gtk_window_add_accel_group(GTK_WINDOW(fui), accelgroup);
 
@@ -308,15 +267,15 @@ static GtkItemFactoryEntry menu_items[] = {
 }
 
 static void
-close_cb(FriendsUI *fui) {
+close_cb(JamFriendsUI *fui) {
 	gtk_widget_destroy(GTK_WIDGET(fui));
 }
 
 static void
-update_edit_menu(FriendsUI *fui, GtkTreeSelection *sel) {
+update_edit_menu(JamFriendsUI *fui, GtkTreeSelection *sel) {
 	GtkTreeModel *model;
 	GtkTreeIter iter;
-	Friend *f = NULL;
+	LJFriend *f = NULL;
 	gboolean issel, active;
 
 	if (sel == NULL)
@@ -329,7 +288,7 @@ update_edit_menu(FriendsUI *fui, GtkTreeSelection *sel) {
 				0, &f,
 				-1);
 
-	active = issel && f->conn & FRIEND_CONN_MY;
+	active = issel && f->conn & LJ_FRIEND_CONN_MY;
 	gtk_widget_set_sensitive(
 			gtk_item_factory_get_widget_by_action(fui->itemfactory,
 				ACTION_EDIT),
@@ -349,12 +308,12 @@ update_edit_menu(FriendsUI *fui, GtkTreeSelection *sel) {
 }
 
 static void 
-friend_add_cb(FriendsUI *fui) {
+friend_add_cb(JamFriendsUI *fui) {
 	GtkTreeSelection *sel;
 	GtkTreeModel *model = NULL;
 	GtkTreeIter iter;
 
-	Friend *f = NULL, *newf;
+	LJFriend *f = NULL, *newf;
 
 	sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(fui->friendview));
 
@@ -363,40 +322,59 @@ friend_add_cb(FriendsUI *fui) {
 				0, &f,
 				-1);
 
-		if (f->conn & FRIEND_CONN_MY) {
+		if (f->conn & LJ_FRIEND_CONN_MY) {
 			/* we can't add friends we already have... */
 			f = NULL;
 		}
 	}
 
-	newf = friend_edit_dlg_run(GTK_WIDGET(fui), FALSE, f);
+	newf = friend_edit_dlg_run(GTK_WINDOW(fui), fui->account, FALSE, f);
 	if (!newf) return;
 
-	if (f == newf || (f = friend_exists(fui, newf->username)) != NULL) {
+	if (f != newf) {
+		/* they didn't edit the friend they might have clicked on,
+		 * but they could have typed an existing friend's name in. */
+		f = friend_exists(fui, newf->username);
+		if (f) {
+			f->foreground = newf->foreground;
+			f->background = newf->background;
+			lj_friend_free(newf);
+			newf = f;
+		}
+	}
+
+	/* did they modify an existing friend, or add a new one? */
+	if (f == newf) {
 		GtkTreePath *path;
 
-		f->conn |= FRIEND_CONN_MY;
+		f->conn |= LJ_FRIEND_CONN_MY;
 
 		/* now we need to signal to the tree model
 		 * that an element has changed. */
-		if (f == newf) {
-			gtk_tree_model_get_iter_first(model, &iter);
-			do {
-				gtk_tree_model_get(model, &iter, 0, &newf, -1);
-				gtk_tree_model_iter_next(model, &iter);
-			} while (newf != f);
-		} else {
-			friend_free(newf);
-		}
+		gtk_tree_model_get_iter_first(model, &iter);
+		do {
+			gtk_tree_model_get(model, &iter, 0, &newf, -1);
+		} while (newf != f && gtk_tree_model_iter_next(model, &iter));
 
-		path = gtk_tree_model_get_path(model, &iter);
-		gtk_tree_model_row_changed(model, path, &iter);
-		gtk_tree_path_free(path);
+		/* it's possible they modified a friend
+		 * who is not in the current view,
+		 * so we only should modify the model
+		 * if we found that friend. */
+		if (newf == f) {
+			path = gtk_tree_model_get_path(model, &iter);
+			gtk_tree_model_row_changed(model, path, &iter);
+			gtk_tree_path_free(path);
+		}
 	} else {
 		GtkListStore *liststore = GTK_LIST_STORE(model);
 		GtkTreeIter iter;
 
-		fui->friends = g_list_insert_sorted(fui->friends, newf, friend_compare);
+		/* if the user adds themself, the connection is immediately two-way. */
+		if (g_ascii_strcasecmp(newf->username,
+					jam_account_get_username(JAM_ACCOUNT(fui->account))) == 0)
+			newf->conn = LJ_FRIEND_CONN_BOTH;
+
+		fui->friends = g_slist_insert_sorted(fui->friends, newf, friend_compare);
 
 		gtk_list_store_append(liststore, &iter);
 		gtk_list_store_set(liststore, &iter,
@@ -408,13 +386,13 @@ friend_add_cb(FriendsUI *fui) {
 }
 
 static void 
-friend_edit_cb(FriendsUI *fui) {
+friend_edit_cb(JamFriendsUI *fui) {
 	GtkTreeSelection *sel;
 	GtkTreeModel *model;
 	GtkTreeIter iter;
 	GtkTreePath *path;
 
-	Friend *f;
+	LJFriend *f;
 
 	sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(fui->friendview));
 
@@ -425,10 +403,10 @@ friend_edit_cb(FriendsUI *fui) {
 			0, &f,
 			-1);
 
-	f = friend_edit_dlg_run(GTK_WIDGET(fui), TRUE, f);
+	f = friend_edit_dlg_run(GTK_WINDOW(fui), fui->account, TRUE, f);
 	if (!f) return;
 
-	f->conn |= FRIEND_CONN_MY;
+	f->conn |= LJ_FRIEND_CONN_MY;
 
 	path = gtk_tree_model_get_path(model, &iter);
 	gtk_tree_model_row_changed(model, path, &iter);
@@ -439,12 +417,12 @@ friend_edit_cb(FriendsUI *fui) {
 }
 
 static void 
-friend_remove_cb(FriendsUI *fui) {
+friend_remove_cb(JamFriendsUI *fui) {
 	GtkTreeSelection *sel;
 	GtkTreeModel *model;
 	GtkTreeIter iter;
 
-	Friend *f;
+	LJFriend *f;
 
 	sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(fui->friendview));
 
@@ -458,15 +436,20 @@ friend_remove_cb(FriendsUI *fui) {
 	if (!request_remove_friend(fui, f->username))
 		return;
 
-	f->conn &= ~FRIEND_CONN_MY;
+	f->conn &= ~LJ_FRIEND_CONN_MY;
+	
+	/* if the user removes themself, the connection is completely gone. */
+	if (g_ascii_strcasecmp(f->username,
+				jam_account_get_username(JAM_ACCOUNT(fui->account))) == 0)
+		f->conn = 0;
+
 	if (f->conn == 0) {
 		/* delete from our list. */
 		gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
-		fui->friends = g_list_remove(fui->friends, f);
-		friend_free(f);
+		fui->friends = g_slist_remove(fui->friends, f);
+		lj_friend_free(f);
 	} else {
 		/* we need to tell the view that we changed something. */
-		/* FIXME: gtk can't possibly be this ugly. :( */
 		GtkTreePath *path;
 
 		path = gtk_tree_model_get_path(model, &iter);
@@ -479,45 +462,35 @@ friend_remove_cb(FriendsUI *fui) {
 }
 
 static void
-friendgroups_cb(FriendsUI *fui) {
-	friendgroups_dialog_new(GTK_WIDGET(fui), fui->friends);
+friendgroups_cb(JamFriendsUI *fui) {
+	friendgroups_dialog_new(GTK_WINDOW(fui), fui->account, fui->friends);
 }
 
 static void
-stats_show_cb(FriendsUI *fui, gint action, GtkCheckMenuItem *item) {
-	conf.options.friends_statsvis = gtk_check_menu_item_get_active(item);
-	lj_widget_set_visible(fui->statspanel, conf.options.friends_statsvis);
-}
-
-static void
-filter_show_cb(FriendsUI *fui, gint action, GtkCheckMenuItem *item) {
-	conf.options.friends_filtervis = gtk_check_menu_item_get_active(item);
-	lj_widget_set_visible(fui->filterpanel, conf.options.friends_filtervis);
+stats_cb(JamFriendsUI *fui, gint action, GtkCheckMenuItem *item) {
+	conf.options.friends_hidestats = gtk_check_menu_item_get_active(item);
+	jam_widget_set_visible(fui->statspanel, !conf.options.friends_hidestats);
 }
 
 static void 
-row_selected(GtkTreeSelection *sel, FriendsUI *fui) {
+row_selected(GtkTreeSelection *sel, JamFriendsUI *fui) {
 	update_edit_menu(fui, sel);
 }
 
 static gboolean
-request_remove_friend(FriendsUI *fui, const char *username) {
-	NetRequest *request;
-	NetResult *result;
+request_remove_friend(JamFriendsUI *fui, const char *username) {
+	NetContext *ctx;
+	LJEditFriends *ef;
+	gboolean ret;
 
-	request = net_request_new("editfriends");
-	net_request_set(request,
-			g_strdup_printf("editfriend_delete_%s", username),
-			g_strdup("1"));
-	result = net_request_run(GTK_WIDGET(fui), _("Deleting Friend..."), request);
-	hash_destroy(request);
+	ef = lj_editfriends_new(jam_account_lj_get_user(fui->account));
+	lj_editfriends_add_delete(ef, username);
+	ctx = net_ctx_gtk_new(GTK_WINDOW(fui), _("Deleting Friend"));
+	ret = net_run_verb_ctx((LJVerb*)ef, ctx, NULL);
+	lj_editfriends_free(ef);
+	net_ctx_gtk_free(ctx);
 
-	if (net_result_succeeded(result)) {
-		net_result_free(result);
-		return TRUE;
-	}
-	net_result_free(result);
-	return FALSE;
+	return ret;
 }
 
 static void
@@ -527,23 +500,23 @@ link_data_func(GtkTreeViewColumn *tree_column,
                GtkTreeIter       *iter, 
                gpointer           data)
 {
-	FriendsUI *fui = data;
-	Friend *friend;
+	JamFriendsUI *fui = data;
+	LJFriend *friend;
 	gtk_tree_model_get(model, iter, 
 			0, &friend,
 			-1);
 	switch (friend->conn) {
-		case FRIEND_CONN_MY:
+		case LJ_FRIEND_CONN_MY:
 			g_object_set(cell,
 					"pixbuf", fui->pb_rarrow,
 					NULL);
 			break;
-		case FRIEND_CONN_OF:
+		case LJ_FRIEND_CONN_OF:
 			g_object_set(cell,
 					"pixbuf", fui->pb_larrow,
 					NULL);
 			break;
-		case FRIEND_CONN_BOTH:
+		case LJ_FRIEND_CONN_BOTH:
 			g_object_set(cell,
 					"pixbuf", fui->pb_lrarrow,
 					NULL);
@@ -557,7 +530,7 @@ link_sort_func(GtkTreeModel *model,
                GtkTreeIter  *b,
                gpointer      data)
 {
-	Friend *fa, *fb;
+	LJFriend *fa, *fb;
 	gtk_tree_model_get(model, a, 
 			0, &fa,
 			-1);
@@ -578,13 +551,13 @@ user_pixbuf_data_func(GtkTreeViewColumn *tree_column,
                       GtkTreeIter       *iter, 
                       gpointer           data)
 {
-	FriendsUI *fui = data;
-	Friend *friend;
+	JamFriendsUI *fui = data;
+	LJFriend *friend;
 	gtk_tree_model_get(model, iter, 
 			0, &friend,
 			-1);
 	switch (friend->type) {
-		case FRIEND_TYPE_COMMUNITY:
+		case LJ_FRIEND_TYPE_COMMUNITY:
 			g_object_set(cell,
 					"pixbuf", fui->pb_comm,
 					NULL);
@@ -596,6 +569,14 @@ user_pixbuf_data_func(GtkTreeViewColumn *tree_column,
 	}
 }
 
+static void
+rgb_to_gdkcolor(guint32 color, GdkColor *gdkcolor) {
+	/* gdkcolors are 16 bits per channel. */
+	gdkcolor->red   = (color & 0x00FF0000) >> 8;
+	gdkcolor->green = (color & 0x0000FF00);
+	gdkcolor->blue  = (color & 0x000000FF) << 8;
+}
+
 static void
 username_data_func(GtkTreeViewColumn *tree_column, 
                     GtkCellRenderer   *cell, 
@@ -603,21 +584,20 @@ username_data_func(GtkTreeViewColumn *tree_column,
                     GtkTreeIter       *iter, 
                     gpointer           data)
 {
-	Friend *friend;
-	GdkColor fg, bg;
+	LJFriend *friend;
+	GdkColor fg = {0}, bg = {0};
 
 	gtk_tree_model_get(model, iter, 
 			0, &friend,
 			-1);
 
-	gdk_color_parse(friend->foreground, &fg);
-	gdk_color_parse(friend->background, &bg);
+	rgb_to_gdkcolor(friend->foreground, &fg);
+	rgb_to_gdkcolor(friend->background, &bg);
 
 	g_object_set(cell,
 			"text", friend->username,
-			/* FIXME: gtk bug?  setting the background color kills the image.
 			"foreground-gdk", &fg,
-			"background-gdk", &bg,*/
+			"background-gdk", &bg,
 			NULL);
 }
 
@@ -627,7 +607,7 @@ username_sort_func(GtkTreeModel *model,
                    GtkTreeIter  *b,
                    gpointer      data)
 {
-	Friend *fa, *fb;
+	LJFriend *fa, *fb;
 	gtk_tree_model_get(model, a, 
 			0, &fa,
 			-1);
@@ -644,7 +624,7 @@ fullname_data_func(GtkTreeViewColumn *tree_column,
                    GtkTreeIter       *iter, 
                    gpointer           data)
 {
-	Friend *friend;
+	LJFriend *friend;
 	gtk_tree_model_get(model, iter, 
 			0, &friend,
 			-1);
@@ -660,7 +640,7 @@ fullname_sort_func(GtkTreeModel *model,
                GtkTreeIter  *b,
                gpointer      data)
 {
-	Friend *fa, *fb;
+	LJFriend *fa, *fb;
 	gtk_tree_model_get(model, a, 
 			0, &fa,
 			-1);
@@ -671,13 +651,13 @@ fullname_sort_func(GtkTreeModel *model,
 }
 
 static gchar*
-selected_username_get(FriendsUI *fui) {
+selected_username_get(JamFriendsUI *fui) {
 	GtkWidget *view = fui->friendview;
 	GtkTreeSelection *sel;
 	GtkTreeModel *model;
 	GtkTreeIter iter;
 
-	Friend *f = NULL;
+	LJFriend *f = NULL;
 
 	sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(view));
 
@@ -690,27 +670,27 @@ selected_username_get(FriendsUI *fui) {
 }
 
 static void
-friend_journal_view_cb(FriendsUI *fui) {
+friend_journal_view_cb(JamFriendsUI *fui) {
 	gchar *friendname = selected_username_get(fui);
 	gchar url[2000];
 
 	g_snprintf(url, 2000, "%s/users/%s/",
-			conf_cur_server()->url, friendname);
-	spawn_url(GTK_WIDGET(fui), url);
+			jam_account_lj_get_server(fui->account)->url, friendname);
+	spawn_url(GTK_WINDOW(fui), url);
 }
 
 static void
-friend_userinfo_view_cb(FriendsUI *fui) {
+friend_userinfo_view_cb(JamFriendsUI *fui) {
 	gchar *friendname = selected_username_get(fui);
 	gchar url[2000];
 
 	g_snprintf(url, 2000, "%s/userinfo.bml?user=%s",
-			conf_cur_server()->url, friendname);
-	spawn_url(GTK_WIDGET(fui), url);
+			jam_account_lj_get_server(fui->account)->url, friendname);
+	spawn_url(GTK_WINDOW(fui), url);
 }
 
 static gboolean
-button_press_cb(GtkTreeView *view, GdkEventButton *e, FriendsUI *fui) {
+button_press_cb(GtkTreeView *view, GdkEventButton *e, JamFriendsUI *fui) {
 	GtkWidget *frmenu;
 
 	if (e->button != 3)
@@ -727,7 +707,7 @@ button_press_cb(GtkTreeView *view, GdkEventButton *e, FriendsUI *fui) {
 static gboolean
 search_equal_cb(GtkTreeModel *model, gint column, const gchar *key,
 		GtkTreeIter *iter, gpointer data) {
-	Friend *f;
+	LJFriend *f;
 	gtk_tree_model_get(model, iter,
 			0, &f,
 			-1);
@@ -735,7 +715,7 @@ search_equal_cb(GtkTreeModel *model, gint column, const gchar *key,
 }
 
 static GtkWidget*
-friends_list_create(FriendsUI *fui) {
+friends_list_create(JamFriendsUI *fui) {
 	GtkWidget *view;
 	GtkTreeViewColumn *column;
 	GtkCellRenderer *cell_renderer;
@@ -764,9 +744,9 @@ friends_list_create(FriendsUI *fui) {
 			G_CALLBACK(row_selected), fui);
 
 	fui->pb_user = gtk_widget_render_icon(view, 
-			"logjam-user", GTK_ICON_SIZE_MENU, NULL);
+			"logjam-ljuser", GTK_ICON_SIZE_MENU, NULL);
 	fui->pb_comm = gtk_widget_render_icon(view, 
-			"logjam-community", GTK_ICON_SIZE_MENU, NULL);
+			"logjam-ljcomm", GTK_ICON_SIZE_MENU, NULL);
 	fui->pb_larrow = icons_larrow_pixbuf();
 	fui->pb_rarrow = icons_rarrow_pixbuf();
 	fui->pb_lrarrow = icons_lrarrow_pixbuf();
@@ -831,7 +811,7 @@ friends_list_create(FriendsUI *fui) {
 }
 
 static GtkWidget*
-make_stats_panel(FriendsUI *fui) {
+make_stats_panel(JamFriendsUI *fui) {
 	GtkWidget *vbox, *label;
 
 	vbox = gtk_vbox_new(FALSE, 5);
@@ -858,9 +838,9 @@ make_stats_panel(FriendsUI *fui) {
 }
 
 static void
-friendsui_destroy_cb(GtkWidget *w, FriendsUI *fui) {
-	g_list_foreach(fui->friends, (GFunc)friend_free, NULL);
-	g_list_free(fui->friends);
+friendsui_destroy_cb(GtkWidget *w, JamFriendsUI *fui) {
+	g_slist_foreach(fui->friends, (GFunc)lj_friend_free, NULL);
+	g_slist_free(fui->friends);
 
 	unref_and_null(fui->pb_user);
 	unref_and_null(fui->pb_comm);
@@ -886,44 +866,44 @@ make_menu_item(GdkPixbuf *pb, const char *label) {
 }
 
 static void
-filter_type_cb(GtkOptionMenu *om, FriendsUI *fui) {
+filter_type_cb(GtkOptionMenu *om, JamFriendsUI *fui) {
 	/* yuck, hard-coded magic numbers. */
 	switch (gtk_option_menu_get_history(om)) {
 		case 0: /* reset */
 			fui->filter_type = 0;
 			break;
 		case 2: 
-			fui->filter_type = FRIEND_TYPE_USER;
+			fui->filter_type = LJ_FRIEND_TYPE_USER;
 			break;
 		case 3: 
-			fui->filter_type = FRIEND_TYPE_COMMUNITY;
+			fui->filter_type = LJ_FRIEND_TYPE_COMMUNITY;
 			break;
 	}
 	populate_model(fui);
 }
 
 static void
-filter_conn_cb(GtkOptionMenu *om, FriendsUI *fui) {
+filter_conn_cb(GtkOptionMenu *om, JamFriendsUI *fui) {
 	/* yuck, hard-coded magic numbers. */
 	switch (gtk_option_menu_get_history(om)) {
 		case 0: /* reset */
 			fui->filter_conn = 0;
 			break;
 		case 2: 
-			fui->filter_conn = FRIEND_CONN_BOTH;
+			fui->filter_conn = LJ_FRIEND_CONN_BOTH;
 			break;
 		case 3: 
-			fui->filter_conn = FRIEND_CONN_MY;
+			fui->filter_conn = LJ_FRIEND_CONN_MY;
 			break;
 		case 4: 
-			fui->filter_conn = FRIEND_CONN_OF;
+			fui->filter_conn = LJ_FRIEND_CONN_OF;
 			break;
 	}
 	populate_model(fui);
 }
 
 static GtkWidget*
-make_filter_box(FriendsUI *fui) {
+make_filter_box(JamFriendsUI *fui) {
 	GtkWidget *box, *menu, *item;;
 
 	fui->omtype = gtk_option_menu_new();
@@ -959,28 +939,22 @@ make_filter_box(FriendsUI *fui) {
 	gtk_box_pack_start(GTK_BOX(box), fui->omtype, FALSE, FALSE, 5);
 	gtk_box_pack_start(GTK_BOX(box), fui->omconn, FALSE, FALSE, 5);
 
-	fui->filterpanel = box;
-
 	return box;
 }
 
 void 
-friends_manager_show(GtkWidget *mainwin) {
-	FriendsUI *fui;
+friends_manager_show(GtkWindow *mainwin, JamAccountLJ *acc) {
+	JamFriendsUI *fui;
 	GtkWidget *mainbox, *hbox;
+	GSList *friends = NULL;
 
-	/* prevent a crash when we get here by hotkey but we're not logged in */
-	if (!conf.loginok)
+	if (!load_friends(mainwin, acc, &friends))
 		return;
 
 	fui = FRIENDSUI(friendsui_new());
-	if (load_friends(fui, mainwin) < 0) {
-		g_free(fui);
-		return;
-	}
-
+	fui->account = acc;
+	fui->friends = friends;
 	gtk_window_set_title(GTK_WINDOW(fui), _("Friends Manager"));
-	lj_win_set_icon(GTK_WIDGET(fui));
 	gtk_window_set_default_size(GTK_WINDOW(fui), 400, 300);
 	geometry_tie(GTK_WIDGET(fui), GEOM_FRIENDS);
 	g_signal_connect(G_OBJECT(fui), "destroy",
@@ -999,14 +973,9 @@ friends_manager_show(GtkWidget *mainwin) {
 
 	gtk_widget_show_all(mainbox);
 
-	gtk_widget_hide(fui->statspanel);
-	gtk_widget_hide(fui->filterpanel);
 	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(
-				gtk_item_factory_get_widget_by_action(fui->itemfactory, ACTION_STATISTICS)),
-			conf.options.friends_statsvis);
-	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(
-				gtk_item_factory_get_widget_by_action(fui->itemfactory, ACTION_FILTER)),
-			conf.options.friends_filtervis);
+				gtk_item_factory_get_widget_by_action(fui->itemfactory, ACTION_HIDE_STATISTICS)),
+			conf.options.friends_hidestats);
 
 	gtk_container_add(GTK_CONTAINER(fui), mainbox);
 
@@ -1014,23 +983,23 @@ friends_manager_show(GtkWidget *mainwin) {
 }
 
 static void
-export_do(FriendsUI *fui, GtkFileSelection *fsel) {
+export_do(JamFriendsUI *fui, GtkFileSelection *fsel) {
 	const char *filename = gtk_file_selection_get_filename(fsel);
 	FILE *fout;
-	Friend *f;
-	GList *l;
+	LJFriend *f;
+	GSList *l;
 
 	if ((fout = fopen(filename, "w")) == NULL) {
-		lj_warning(GTK_WIDGET(fui), _("Unable to open %s: %s\n"), 
-				filename, strerror(errno));
+		jam_warning(GTK_WINDOW(fui), _("Unable to open %s: %s\n"), 
+				filename, g_strerror(errno));
 		return;
 	}
 	for (l = fui->friends; l != NULL; l = l->next) {
 		f = l->data;
 
 		fprintf(fout, "%c-%c %s\n", 
-				(f->conn & FRIEND_CONN_OF ? '<' : ' '),
-				(f->conn & FRIEND_CONN_MY ? '>' : ' '),
+				(f->conn & LJ_FRIEND_CONN_OF ? '<' : ' '),
+				(f->conn & LJ_FRIEND_CONN_MY ? '>' : ' '),
 				f->username);
 	}
 
@@ -1053,12 +1022,11 @@ suggest_cb(GtkWidget *w, GtkWidget *entry) {
 }
 
 static void
-export_cb(FriendsUI *fui) {
+export_cb(JamFriendsUI *fui) {
 	GtkWidget *fsel;
 	GtkWidget *bstamp;
 
-	fsel = gtk_file_selection_new(_("Select output file"));
-	gtk_window_set_transient_for(GTK_WINDOW(fsel), GTK_WINDOW(fui));
+	fsel = jam_filesel_new(_("Select output file"), GTK_WINDOW(fui));
 
 	bstamp = gtk_button_new_with_label(_("  Suggest Filename  "));
 	gtk_box_pack_start(GTK_BOX(GTK_FILE_SELECTION(fsel)->action_area),
@@ -1068,7 +1036,6 @@ export_cb(FriendsUI *fui) {
 			GTK_FILE_SELECTION(fsel)->selection_entry);
 	gtk_widget_show(bstamp);
 
-	lj_win_set_icon(fsel);
 	if (gtk_dialog_run(GTK_DIALOG(fsel)) == GTK_RESPONSE_OK)
 		export_do(fui, GTK_FILE_SELECTION(fsel));
 	gtk_widget_destroy(fsel);
