@@ -1,20 +1,22 @@
 /* logjam - a GTK client for LiveJournal.
- * Copyright (C) 2000-2002 Evan Martin <evan@livejournal.com>
+ * Copyright (C) 2000-2003 Evan Martin <evan@livejournal.com>
  *
  * vim: tabstop=4 shiftwidth=4 noexpandtab :
- * $Id: history.c,v 1.19 2002/10/03 01:11:43 martine Exp $
  */
 
-#include <gtk/gtk.h>
+#include "gtk-all.h"
 #include <time.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
+#include <livejournal/getevents.h>
+
 #include "util.h"
 #include "conf.h"
 #include "network.h"
 #include "spawn.h"
+#include "history.h"
 
 #if 0
 typedef struct {
@@ -154,16 +156,16 @@ static GtkWidget*
 make_buttonbox(history_calendar_dlg *hcdlg) {
 	GtkWidget *box, *button;
 
-	box = lj_dialog_buttonbox_new();
+	box = jam_dialog_buttonbox_new();
 	button = gtk_button_new_with_label(" Edit... ");
 	g_signal_connect(G_OBJECT(button), "clicked",
 			G_CALLBACK(hc_edit_cb), hcdlg);
-	lj_dialog_buttonbox_add(box, button);
+	jam_dialog_buttonbox_add(box, button);
 
 	/*button = web_button(hcdlg->win, "View...");
 	g_signal_connect(G_OBJECT(button), "clicked",
 			G_CALLBACK(hc_web_cb), hcdlg);
-	lj_dialog_buttonbox_add(box, button);*/
+	jam_dialog_buttonbox_add(box, button);*/
 	return box;
 }
 
@@ -172,16 +174,16 @@ void history_dialog(GtkWidget *mainwin) {
 
 	hcdlg = g_new0(history_calendar_dlg, 1);
 
-	hcdlg->win = lj_dialog_new(mainwin, "History Overview", 400, -1);
+	hcdlg->win = jam_dialog_new(mainwin, "History Overview", 400, -1);
 	g_signal_connect(G_OBJECT(hcdlg->win), "destroy",
 			G_CALLBACK(free_dlg_cb), hcdlg);
 
 	hcdlg->bbox = make_buttonbox(hcdlg);
-	lj_dialog_set_contents_buttonbox(hcdlg->win,
+	jam_dialog_set_contents_buttonbox(hcdlg->win,
 			make_contents(hcdlg),
 			hcdlg->bbox);
 
-	lj_dialog_add_close(hcdlg->win);
+	jam_dialog_add_close(hcdlg->win);
 	gtk_widget_set_sensitive(hcdlg->bbox, FALSE);
 	gtk_widget_show(hcdlg->win);
 
@@ -230,7 +232,7 @@ getdaycounts_run(history_calendar_dlg* hcdlg) {
 	NetRequest *request;
 	NetResult  *result;
 	
-	request = net_request_new("getdaycounts");
+	request = XXX("getdaycounts");
 	
 	result = net_request_run(hcdlg->win, "Loading calendar...", request);
 	net_request_free(request);
@@ -269,7 +271,7 @@ change_day(history_calendar_dlg *hcdlg) {
 	
 	if (hcdlg->firstyear == 0) return; /* this can happen if they cancelled the getdaycounts */
 
-	request = net_request_new("getevents");
+	request = XXX("getevents");
 
 	net_request_seti(request,  "truncate",      50);
 	net_request_seti(request,  "prefersubject", 1);
@@ -302,7 +304,7 @@ change_day(history_calendar_dlg *hcdlg) {
 		append[1] = "[no events]";
 		gtk_clist_append(GTK_CLIST(hcdlg->evlist), append);
 		gtk_clist_set_selectable(GTK_CLIST(hcdlg->evlist), 0, FALSE);
-		hash_destroy(result);
+		net_result_free(result);
 		return;
 	}
 
@@ -393,7 +395,7 @@ hc_edit_cb(GtkWidget *w, history_calendar_dlg *hcdlg) {
 history_delete_selected(history_item_dlg *hidlg) {
 	GHashTable *request, *result;
 	
-	request = net_request_new("editevent");
+	request = XXX("editevent");
 
 	g_hash_table_insert(request, g_strdup("itemid"),
 			g_strdup_printf("%d", hidlg->itemid));
@@ -415,7 +417,7 @@ hc_web_cb(GtkWidget *w, history_calendar_dlg *hcdlg) {
 	char *spawn;
 
 	spawn = g_strdup_printf("%s/talkread.bml?itemid=%d", 
-			conf_cur_server()->url,
+			c_cur_server()->url,
 			hc_current_itemid(hcdlg));
 	spawn_url(spawn);
 	g_free(spawn);
@@ -489,17 +491,17 @@ static GHashTable*
 hi_request_run(history_item_dlg *hidlg, int itemid) {
 	GHashTable *request, *result;
 
-	request = net_request_new("getevents");
+	request = XXX("getevents");
 
 	g_hash_table_insert(request, g_strdup("selecttype"), g_strdup("one"));
 	g_hash_table_insert(request, g_strdup("lineendings"), g_strdup("unix"));
 	g_hash_table_insert(request, g_strdup("itemid"), g_strdup_printf("%d", itemid));
 
 	result = net_request_run(hidlg->win, "Loading event...", request);
-	hash_destroy(request);
+	net_request_destroy(request);
 
 	if (!net_request_succeeded(result)) {
-		hash_destroy(result);
+		net_result_free(result);
 		return NULL;
 	}
 	return result;
@@ -512,9 +514,9 @@ hi_load_result(history_item_dlg *hidlg, GHashTable* result) {
 
 	value = g_hash_table_lookup(result, "events_count");
 	if (value == NULL || strcmp(value, "1") != 0) {
-		lj_messagebox(hidlg->win, "Error Loading Event",
+		jam_messagebox(hidlg->win, "Error Loading Event",
 				"Event not found.");
-		hash_destroy(result);
+		net_result_free(result);
 		return FALSE;
 	}
 
@@ -539,7 +541,7 @@ hi_load_result(history_item_dlg *hidlg, GHashTable* result) {
 	hi_load_datetime(hidlg, result);
 	hi_load_security(hidlg, result);
 	hi_load_metadata(hidlg, result);
-	hash_destroy(result);
+	net_result_free(result);
 	return TRUE;
 }
 
@@ -547,7 +549,7 @@ static void
 hi_save_cb(GtkWidget *w, history_item_dlg *hidlg) {
 	GHashTable *request, *result;
 
-	request = net_request_new("editevent");
+	request = XXX("editevent");
 
 	g_hash_table_insert(request, g_strdup("itemid"),
 			g_strdup_printf("%d", hidlg->itemid));
@@ -570,15 +572,15 @@ hi_save_cb(GtkWidget *w, history_item_dlg *hidlg) {
 	security_append_to_request(&hidlg->security, request);
 
 	result = net_request_run(hidlg->win, "Saving Item...", request);
-	hash_destroy(request);
+	net_request_free(request);
 
-	hash_destroy(result);
+	net_result_free(result);
 }
 
 static void
 hi_delete_cb(GtkWidget *w, history_item_dlg *hidlg) {
 	if (conf.options.confirmdelete)
-		if (!lj_confirm(hidlg->win, "Delete?", "Delete this entry?"))
+		if (!jam_confirm(hidlg->win, "Delete?", "Delete this entry?"))
 			return;
 
 	history_delete_selected(hidlg);
@@ -647,7 +649,7 @@ hi_dialog_run(GtkWidget *parent, int itemid) {
 	if (result == NULL) 
 		return 0;
 
-	hidlg->win = lj_dialog_new(parent, "History Item", 300, 300);
+	hidlg->win = jam_dialog_new(parent, "History Item", 300, 300);
 	geometry_tie(hidlg->win, GEOM_HISTORY_ITEM);
 	g_signal_connect(G_OBJECT(hidlg->win), "destroy",
 			G_CALLBACK(gtk_main_quit), NULL);
@@ -702,19 +704,19 @@ hi_dialog_run(GtkWidget *parent, int itemid) {
 		gtk_box_pack_end(GTK_BOX(vbox), hidlg->metamgr, FALSE, FALSE, 0);
 	}
 	
-	lj_dialog_set_contents(hidlg->win, vbox);
+	jam_dialog_set_contents(hidlg->win, vbox);
 
 	button = gtk_button_new_with_label("  Save Changes  ");
 	g_signal_connect(G_OBJECT(button), "clicked",
 			G_CALLBACK(hi_save_cb), (gpointer) hidlg);
-	lj_dialog_add_button(hidlg->win, button);
+	jam_dialog_add_button(hidlg->win, button);
 
 	button = gtk_button_new_with_label("  Delete  ");
 	g_signal_connect(G_OBJECT(button), "clicked",
 			G_CALLBACK(hi_delete_cb), (gpointer) hidlg);
-	lj_dialog_add_button(hidlg->win, button);
+	jam_dialog_add_button(hidlg->win, button);
 	
-	lj_dialog_add_close(hidlg->win);
+	jam_dialog_add_close(hidlg->win);
 
 	hi_load_result(hidlg, result);
 
@@ -728,37 +730,33 @@ hi_dialog_run(GtkWidget *parent, int itemid) {
 }
 #endif
 
-Entry*
-history_load_itemid(GtkWidget *win, int itemid) {
-	NetRequest *request;
-	NetResult *result;
-	Entry *entry;
+LJEntry*
+history_load_itemid(GtkWindow *parent, JamAccount *acc, const char *usejournal, int itemid) {
+	NetContext *ctx;
+	LJGetEventsSingle *getevents;
+	LJEntry *entry;
 
-	/* FIXME cache: hit cache first. */
-	request = net_request_new("getevents");
-	net_request_copys(request, "selecttype", "one");
-	net_request_seti(request, "itemid", itemid);
-	net_request_copys(request, "lineendings", "unix");
-	if (conf.usejournal)
-		net_request_copys(request, "usejournal", conf.usejournal);
-
-	result = net_request_run(win, _("Loading Entry..."), request);
-	net_request_free(request);
-
-	if (!net_result_succeeded(result)) {
-		net_result_free(result);
+	if (!JAM_ACCOUNT_IS_LJ(acc)) {
+		g_warning("XXX blogger: history for blogger\n");
 		return NULL;
 	}
 
-	entry = entry_from_result(result, 1);
-	net_result_free(result);
+	getevents = lj_getevents_single_new(jam_account_lj_get_user(JAM_ACCOUNT_LJ(acc)), usejournal, itemid);
+	ctx = net_ctx_gtk_new(parent, _("Loading Entry"));
+	if (!net_run_verb_ctx((LJVerb*)getevents, ctx, NULL)) {
+		lj_getevents_single_free(getevents, TRUE);
+		net_ctx_gtk_free(ctx);
+		return NULL;
+	}
+	entry = getevents->entry;
+	lj_getevents_single_free(getevents, FALSE);
+	net_ctx_gtk_free(ctx);
 
 	return entry;
 }
 
-Entry*
-history_load_latest(GtkWidget *win) {
-	/* FIXME cache: hit cache first. */
-	return history_load_itemid(win, -1);
+LJEntry*
+history_load_latest(GtkWindow *win, JamAccount *acc, const char *usejournal) {
+	return history_load_itemid(win, acc, usejournal, -1);
 }
 
