@@ -802,6 +802,13 @@ jam_view_settings_changed(JamView *view) {
 #endif
 	if (conf.uifont)
 		jam_widget_set_font(view->entry, conf.uifont);
+
+	if (view->musicbar) {
+	    /* There are many cleaner ways to add/remove the Refresh button when
+	     * the music preference has changed, but this works fine. */
+		music_remove(view);
+		music_add(view);
+	}
 }
 
 GType
