@@ -1,13 +1,12 @@
 /* logjam - a GTK client for LiveJournal.
- * Copyright (C) 2000-2002 Evan Martin <evan@livejournal.com>
+ * Copyright (C) 2000-2003 Evan Martin <evan@livejournal.com>
  *
  * vim: tabstop=4 shiftwidth=4 noexpandtab :
- * $Id: security.c,v 1.9 2002/10/27 00:18:57 martine Exp $
  */
 
 #include "config.h"
 
-#include <gtk/gtk.h>
+#include "gtk-all.h"
 
 #include <stdlib.h>
 
@@ -19,8 +18,9 @@
 
 struct _SecMgr {
 	GtkOptionMenu omenu;
-	Security security;
+	LJSecurity security;
 	GtkWidget *custom;
+	JamAccountLJ *account;
 };
 
 static void secmgr_init(SecMgr *sm);
@@ -28,6 +28,8 @@ static void secmgr_init(SecMgr *sm);
 static void security_changed(SecMgr *sm);
 static void secmgr_destroyed_cb(GObject* sm);
 
+static const LJSecurity security_private = { LJ_SECURITY_PRIVATE, 0 };
+
 /* gtk stuff */
 GType
 secmgr_get_type(void) {
@@ -76,7 +78,7 @@ secmgr_init(SecMgr *sm) {
 	menu = gtk_menu_new();
 	ms = GTK_MENU_SHELL(menu);
 
-	add_menu_item(ms, "logjam-user", _("Public"));
+	add_menu_item(ms, "logjam-ljuser", _("Public"));
 	add_menu_item(ms, "logjam-protected", _("Friends"));
 	add_menu_item(ms, "logjam-private", _("Private"));
 	sm->custom = add_menu_item(ms, NULL, _("Custom..."));
@@ -116,17 +118,26 @@ secmgr_new(gboolean withcustom) {
 }
 
 void
-secmgr_custom_set_sensitive(SecMgr *sm, gboolean sensitive) {
-	gtk_widget_set_sensitive(sm->custom, sensitive);
+secmgr_set_account(SecMgr *sm, JamAccountLJ *account) {
+	gtk_widget_set_sensitive(sm->custom, account != NULL);
+	if (account != sm->account) {
+		LJSecurity oldsec;
+
+		sm->account = account;
+
+		secmgr_security_get(sm, &oldsec);
+		if (oldsec.type == LJ_SECURITY_CUSTOM)
+			secmgr_security_set(sm, &security_private);
+	}
 }
 
 void
-secmgr_security_set(SecMgr *secmgr, const Security *security) {
+secmgr_security_set(SecMgr *secmgr, const LJSecurity *security) {
 	secmgr->security = *security;
 	gtk_option_menu_set_history(GTK_OPTION_MENU(secmgr), secmgr->security.type);
 }
 void
-secmgr_security_get(SecMgr *secmgr, Security *security) {
+secmgr_security_get(SecMgr *secmgr, LJSecurity *security) {
 	*security = secmgr->security;
 }
 
@@ -137,7 +148,7 @@ secmgr_security_get(SecMgr *secmgr, Security *security) {
  * the "changed" handler has already dealt with this change, or the input is
  * known to be good for some other reason. */
 void
-secmgr_security_set_force(SecMgr *secmgr, const Security *security) {
+secmgr_security_set_force(SecMgr *secmgr, const LJSecurity *security) {
 	g_signal_handlers_block_by_func(G_OBJECT(secmgr),
 			security_changed, NULL);
 	secmgr_security_set(secmgr, security);
@@ -148,19 +159,19 @@ secmgr_security_set_force(SecMgr *secmgr, const Security *security) {
 void
 security_changed(SecMgr *sm) {
 	sm->security.type = gtk_option_menu_get_history(GTK_OPTION_MENU(sm));
-	if (sm->security.type == SECURITY_CUSTOM) {
+	if (sm->security.type == LJ_SECURITY_CUSTOM) {
 		sm->security.allowmask = 
 			custom_security_dlg_run(
-					gtk_widget_get_toplevel(GTK_WIDGET(sm)), 
-					sm->security.allowmask);
+					GTK_WINDOW(gtk_widget_get_toplevel(GTK_WIDGET(sm))), 
+					sm->security.allowmask,
+					sm->account);
 	} else {
 		sm->security.allowmask = 0; /* reset it. */
 	}
 
-	if (sm->security.type == SECURITY_CUSTOM 
+	if (sm->security.type == LJ_SECURITY_CUSTOM 
 			&& sm->security.allowmask == 0) {
-		sm->security.type = SECURITY_PRIVATE;
-		gtk_option_menu_set_history(GTK_OPTION_MENU(sm), SECURITY_PRIVATE); /* fixme */
+		secmgr_security_set(sm, &security_private);
 	}
 }
 
@@ -192,18 +203,18 @@ included_toggle_cb(GtkCellRendererToggle *cell,
 }
 
 static GtkTreeModel*
-build_model(guint32 mask) {
+build_model(guint32 mask, LJUser *user) {
 	GtkListStore *store;
-	GList *l;
-	FriendGroup *fg;
+	GSList *l;
+	LJFriendGroup *fg;
 	GtkTreeIter iter;
 
 	store = gtk_list_store_new(3, 
 			G_TYPE_BOOLEAN, /* included?   */
 			G_TYPE_STRING,  /* name        */
 			G_TYPE_POINTER  /* friendgroup */);
-	for (l = conf_cur_user()->friendgroups; l != NULL; l = l->next) {
-		fg = (FriendGroup*)l->data;
+	for (l = user->friendgroups; l != NULL; l = l->next) {
+		fg = (LJFriendGroup*)l->data;
 		gtk_list_store_append(store, &iter);
 		gtk_list_store_set(store, &iter,
 				COL_INCLUDED, ((1L << fg->id) & mask) != 0,
@@ -244,42 +255,44 @@ static guint32
 get_new_mask(GtkTreeModel *model) {
 	GtkTreeIter iter;
 	gboolean included;
-	FriendGroup *fg;
+	LJFriendGroup *fg;
 	guint32 newmask = 0;
 
-	gtk_tree_model_get_iter_first(model, &iter);
-	do {
-		gtk_tree_model_get(model, &iter,
-				COL_INCLUDED, &included,
-				COL_FG, &fg,
-				-1);
-		if (included)
-			newmask |= (1L << fg->id);
-	} while (gtk_tree_model_iter_next(model, &iter));
+	if (gtk_tree_model_get_iter_first(model, &iter)) {
+		do {
+			gtk_tree_model_get(model, &iter,
+					COL_INCLUDED, &included,
+					COL_FG, &fg,
+					-1);
+			if (included)
+				newmask |= (1L << fg->id);
+		} while (gtk_tree_model_iter_next(model, &iter));
+	}
 	return newmask;
 }
 
 guint32
-custom_security_dlg_run(GtkWidget *parent, guint32 mask) {
+custom_security_dlg_run(GtkWindow *parent, guint32 mask, JamAccountLJ *acc) {
 	GtkWidget *dlg;
 	GtkTreeModel *model;
 	GtkWidget *view;
+	LJUser *user = jam_account_lj_get_user(acc);
 
-	lj_warning(parent, "This is an experimental feature.  "
-			"Use custom security at your own peril, "
-			"but please tell us about any bugs.");
-	dlg = lj_dialog_new(parent, "Select Friend Groups", 250, -1);
+	dlg = gtk_dialog_new_with_buttons(_("Select Friend Groups"),
+			parent, GTK_DIALOG_MODAL,
+			GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
+			NULL);
+	jam_win_set_size(GTK_WINDOW(dlg), 300, -1);
 
-	model = build_model(mask);
+	model = build_model(mask, user);
 	view = build_view(model);
 
-	lj_dialog_set_contents(dlg, scroll_wrap(view));
-	lj_dialog_add_okcancel(dlg, NULL);
+	jam_dialog_set_contents(GTK_DIALOG(dlg), scroll_wrap(view));
 
-	if (lj_dialog_run(dlg)) {
-		mask = get_new_mask(model);
-		gtk_widget_destroy(dlg);
-	}
+	gtk_dialog_run(GTK_DIALOG(dlg));
+	mask = get_new_mask(model);
 	g_object_unref(G_OBJECT(model));
+	gtk_widget_destroy(dlg);
 	return mask;
 }
+
