@@ -23,6 +23,7 @@
 #include "smartquotes.h"
 #include "tie.h"
 #include "account.h"
+#include "jamview.h"
 
 /* what's this?  all of these funny structures in the settings box?
  * well, instead of creating and tearing down all of these widgets, i
@@ -543,14 +544,19 @@ proxysettings(void) {
 }
 
 static GtkWidget*
-programsettings(void) {
+programsettings(JamWin *jw) {
 	GtkWidget *group;
 	GtkWidget *button, *hbox;
-
+	SettingsWidget *sw;
+	JamView *jv = jam_win_get_cur_view(jw);
+	
 	group = groupedbox_new_with_text(_("Programs"));
 	groupedbox_pack(GROUPEDBOX(group), sw_make("web_spawn"), FALSE);
 
 	groupedbox_pack(GROUPEDBOX(group), sw_make("music_command"), FALSE);
+	sw = sw_lookup("music_command");
+	g_signal_connect_swapped(G_OBJECT(sw->widget), "changed",
+			G_CALLBACK(jam_view_settings_changed), jv);
 
 	button = gtk_button_new_with_mnemonic(_("_Diagnose"));
 	g_signal_connect(G_OBJECT(button), "clicked",
@@ -563,12 +569,12 @@ programsettings(void) {
 }
 
 static GtkWidget*
-systemsettings(void) {
+systemsettings(JamWin *jw) {
 	GtkWidget *vbox;
 
 	vbox = gtk_vbox_new(FALSE, 18);
 	gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);
-	gtk_box_pack_start(GTK_BOX(vbox), programsettings(), FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox), programsettings(jw), FALSE, FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(vbox), proxysettings(), FALSE, FALSE, 0);
 	return vbox;
 }
@@ -725,7 +731,7 @@ run_settings_dialog(JamWin *jw) {
 			uisettings(jw), gtk_label_new_with_mnemonic(_("Interface")));
 #ifndef G_OS_WIN32
 	gtk_notebook_append_page(GTK_NOTEBOOK(nb), 
-			systemsettings(), gtk_label_new_with_mnemonic(_("System")));
+			systemsettings(jw), gtk_label_new_with_mnemonic(_("System")));
 #endif /* G_OS_WIN32 */
 	if (JAM_ACCOUNT_IS_LJ(jw->account))
 		gtk_notebook_append_page(GTK_NOTEBOOK(nb), 
